#! /usr/bin/env python
# ______________________________________________________________________________
"""pcmesher -H <val>

This script is intended to produce a Dolfin mesh for 4-1 planar contraction.

usage:
  -H <val> refinement of boundary at the rentrant corner
  -h    Print this usage message
"""

# _____________________________________________________________________________
# Module Copyright and license
__author__ = "Andy R Terrel (aterrel@uchicago.edu)"
__date__ = "2008-11"
__copyright__ = "Copyright (C) 2008 Andy R Terrel"
__license__ = "GNU LGPL Version 3.0 or later"

# _____________________________________________________________________________
# Module Imports
import sys
import os
import getopt

# ______________________________________________________________________________
# Module Functions
def createGEO(LC,FILE):
    points = [ (0, 0, LC*4),
               (0, 4, LC*4),
               (3, 4, LC*1.5),
               (3, 1, LC),
               (6, 1, LC*3),
               (6, 0, LC*3),
               (3, 0, LC*2),
             ]

    geo_fp = open(FILE+".geo","w")

    for i,pt in enumerate(points):
        geo_fp.write("Point(%d) = {%f, %f, 0.0, %f};\n" % (i+1,pt[0], pt[1], pt[2]))

    # PT 1,2,3,4  5,6,7,8
    # Lines 1 (1,2); 2(2,3); 3(3,4); 4(4,1)
    #       5 (5,6); 6(6,7); 7(7,8); 8(8,5)
    #
    #
    #
    j = 0
    for i in xrange(len(points)-1):
        j += 1
        geo_fp.write("Line(%d) = {%d, %d};\n" % (j, j, j+1))
    j += 1
    geo_fp.write("Line(%d) = {%d, %d};\n" % (j,j,1))

    j+=1
    geo_fp.write("Line Loop(%d) = {%s};\n" %
                 (j,",".join(map(str, range(1,len(points)+1)))))
    j += 1
    geo_fp.write("Plane Surface(%d) = {%d};\n" % (j, j-1))
    geo_fp.close()

# _____________________________________________________________________________
def main(*args):
    H = None
    try:
        opts, remainder = getopt.getopt(args, "H:h")
    except getopt.GetoptError:
        print "Unrecognized options"
        print __doc__
        sys.exit(1)
    for opt_key, opt_val in opts:
        if opt_key == "-H":
            H = float(opt_val)
        elif opt_key == "-h":
            print __doc__
    if not H:
        print "Please specify a length scale"
        print __doc__
        sys.exit(1)

    FILE = "pc_h_%.3f" %(H)
    FILE = FILE.replace(".","_")
    createGEO(H,FILE)
    os.system("gmsh %s.geo -2" % FILE)
    os.system("dolfin-convert %s.msh %s.xml" % (FILE,FILE))
    return

# ______________________________________________________________________________
# Module Script Runner
if __name__ == "__main__":
    main(*sys.argv[1:])
# ______________________________________________________________________________
# End of mesher.py
