// Copyright (C) 2008-2009 Andy R Terrel
// Licensed under the GNU LGPL Version 3.0 or later.
//
// First added:  2008-09-12
// Last changed: 2009-12-08

#ifndef __GRADE2SOLVER_H
#define __GRADE2SOLVER_H

#include <dolfin.h>
#include "Fluid.h"
#include "StokesSolver.h"

#include "forms/grade2/TaylorHood/P2THG2Vel.h"
#include "forms/grade2/TaylorHood/P2THG2Trans.h"
#include "forms/grade2/TaylorHood/P2THG2Stab.h"
#include "forms/grade2/TaylorHood/P2THG2Res.h"

#include "forms/norms/P2VecL2Diff.h"
#include "forms/norms/P2L2Diff.h"
#include "forms/norms/P2VecL2.h"
#include "forms/norms/P2L2.h"

#include "forms/projections/P2VecDiff.h"
#include "forms/projections/P2Diff.h"
#include "forms/projections/P1Diff.h"

namespace dolfin
{

  class Grade2Solver : public StokesSolver
  {
  public:

    Grade2Solver ();

    Grade2Solver (const Grade2Solver& that);

    ~Grade2Solver ();

    Grade2Solver& operator= (const Grade2Solver& that);

    void solve (Fluid& fluid, GenericFunction& f, bool verbose=false);

    void solve (Fluid& fluid, GenericFunction& f,
		GenericFunction& v_ex, GenericFunction& p_ex, GenericFunction& z_ex);

  private:
    // Managed pointers
    Function* _uu;
    Function* _z;
    Function* _tau;
    GenericFunction* f_alpha;
    GenericFunction* f_mu;
    GenericFunction* f_rho;
    GenericFunction* f_w;
    CellSize* f_h;

    FunctionSpace* _trans_funspace;
    FunctionSpace* _stab_funspace;
    Form* _a_transport;
    Form* _L_transport;
    Form* _a_stab;
    Form* _L_stab;
    Form* _L_res;

    // Unmanaged pointers
    GenericFunction* _v_ex;
    GenericFunction* _p_ex;
    GenericFunction* _z_ex;

    // Private functions
    void deletePtrs ();
    void zeroPtrs();
    void formFactory (Fluid& fluid, GenericFunction& f);
  };
}

#endif
