// Copyright (C) 2008--2009 Andy R Terrel
// Licensed under the GNU LGPL Version 3.0 or later.
//
// First added:   2008-09-12
// Last modified: 2009-12-08


#ifndef __STOKESSOLVER_H
#define __STOKESSOLVER_H

#include <dolfin.h>
#include "Fluid.h"
#include "forms/Stokes/TaylorHood/P2THStokes.h"
#include "forms/Stokes/ScottVogelius/P4SVStokes.h"
#include "forms/norms/P4VecDivL2.h"
#include "forms/projections/P4VecDiv.h"

namespace dolfin
{

  class StokesSolver : public Variable
  {
  public:

    StokesSolver ();

    StokesSolver (const StokesSolver& that);

    ~StokesSolver ();

    StokesSolver& operator= (const StokesSolver& that);

    virtual void solve (Fluid& fluid, GenericFunction& f);

  protected:

    FunctionSpace* _funspace;
    Form* _a_stokes;
    Form* _L_stokes;
    Form* _a_div_proj;
    Form* _L_div_proj;

    virtual void deletePtrs ();

    void mixedMethodSolve (Fluid& fluid, std::vector<const BoundaryCondition *>& bcs,
			   GenericFunction& f);

    void scottVogeliusSolve (Fluid& fluid, std::vector<const BoundaryCondition *>& bcs,
			     GenericFunction& f, Function& f_w);
  };
}

#endif
